@@ -16,21 +16,25 @@ class ActionText::ModelTest < ActiveSupport::TestCase
   end
 
   test "without content" do
-    message = Message.create!(subject: "Greetings")
-    assert message.content.nil?
-    assert message.content.blank?
-    assert message.content.empty?
-    assert_not message.content?
-    assert_not message.content.present?
+    assert_difference("ActionText::RichText.count" => 0) do
+      message = Message.create!(subject: "Greetings")
+      assert_predicate message.content, :nil?
+      assert_predicate message.content, :blank?
+      assert_predicate message.content, :empty?
+      assert_not message.content?
+      assert_not message.content.present?
+    end
   end
 
   test "with blank content" do
-    message = Message.create!(subject: "Greetings", content: "")
-    assert_not message.content.nil?
-    assert message.content.blank?
-    assert message.content.empty?
-    assert_not message.content?
-    assert_not message.content.present?
+    assert_difference("ActionText::RichText.count" => 1) do
+      message = Message.create!(subject: "Greetings", content: "")
+      assert_not message.content.nil?
+      assert_predicate message.content, :blank?
+      assert_predicate message.content, :empty?
+      assert_not message.content?
+      assert_not message.content.present?
+    end
   end
 
   test "embed extraction" do
@@ -99,19 +103,65 @@ class ActionText::ModelTest < ActiveSupport::TestCase
   test "eager loading" do
     Message.create!(subject: "Subject", content: "<h1>Content</h1>")
 
-    message = assert_queries(2) { Message.with_rich_text_content.last }
+    message = assert_queries_count(2) { Message.with_rich_text_content.last }
     assert_no_queries do
       assert_equal "Content", message.content.to_plain_text
     end
   end
 
   test "eager loading all rich text" do
-    Message.create!(subject: "Subject", content: "<h1>Content</h1>", body: "<h2>Body</h2>")
+    2.times do
+      Message.create!(subject: "Subject", content: "<h1>Content</h1>", body: "<h2>Body</h2>")
+    end
+
+    message = assert_queries_count(3) do
+      # 3 queries:
+      # messages x 1
+      # action texts (content) x 1
+      # action texts (body) x 1
+      Message.with_all_rich_text.to_a.last
+    end
 
-    message = assert_queries(1) { Message.with_all_rich_text.last }
     assert_no_queries do
       assert_equal "Content", message.content.to_plain_text
       assert_equal "Body", message.body.to_plain_text
     end
   end
+
+  test "with blank content and store_if_blank: false" do
+    assert_difference("ActionText::RichText.count" => 0) do
+      message = MessageWithoutBlanks.create!(subject: "Greetings", content: "")
+      assert_predicate message.content, :nil?
+      assert_predicate message.content, :blank?
+      assert_predicate message.content, :empty?
+      assert_not message.content?
+      assert_not message.content.present?
+    end
+  end
+
+  test "if allowing blanks, updates rich text record on edit" do
+    message = Message.create!(subject: "Greetings", content: "content")
+    assert_difference("ActionText::RichText.count" => 0) do
+      message.update(content: "")
+    end
+  end
+
+  test "if disallowing blanks, deletes rich text record on edit" do
+    message = MessageWithoutBlanks.create!(subject: "Greetings", content: "content")
+    assert_difference("ActionText::RichText.count" => -1) do
+      message.update(content: "")
+    end
+  end
+
+  test "if disallowing blanks, can still validate presence" do
+    message1 = MessageWithoutBlanksWithContentValidation.new(subject: "Greetings", content: "")
+    assert_not_predicate message1, :valid?
+    message1.content = "content"
+    assert_predicate message1, :valid?
+
+    message2 = MessageWithoutBlanksWithContentValidation.new(subject: "Greetings", content: "content")
+    assert_predicate message2, :valid?
+    message2.content = ""
+    assert_not_predicate message2, :valid?
+  end
 end
