@@ -565,7 +565,8 @@ class QueryCacheTest < ActiveRecord::TestCase
     assert_not_predicate Task, :connected?
 
     Task.cache do
-      assert_queries_count(1) { Task.find(1); Task.find(1) }
+      assert_queries_count(1) { Task.find(1) }
+      assert_no_queries { Task.find(1) }
     ensure
       ActiveRecord::Base.establish_connection(original_connection)
     end
@@ -731,28 +732,16 @@ class QueryCacheTest < ActiveRecord::TestCase
       ActiveRecord::Base.lease_connection.enable_query_cache!
       assert_cache :clean
 
-      main_thread_cache = ActiveRecord::Base.lease_connection.query_cache
-      assert_same main_thread_cache, ActiveRecord::Base.lease_connection.query_cache
-
       thread_a = Thread.new do
         middleware { |env|
           assert_cache :clean
-
-          # In a background thread, the cache instance must stay consistent but be different from the main
-          # thread.
-          background_thread_cache = ActiveRecord::Base.lease_connection.query_cache
-          assert_same background_thread_cache, ActiveRecord::Base.lease_connection.query_cache
-          assert_not_same main_thread_cache, ActiveRecord::Base.lease_connection.query_cache
           [200, {}, nil]
         }.call({})
       end
 
       thread_a.join
-
-      assert_same main_thread_cache, ActiveRecord::Base.lease_connection.query_cache
     ensure
       ActiveRecord::Base.connection_pool.unpin_connection!
-      assert_same main_thread_cache, ActiveRecord::Base.lease_connection.query_cache
     end
   end
 
@@ -769,7 +758,7 @@ class QueryCacheTest < ActiveRecord::TestCase
 
       thread_a = Thread.new do
         middleware { |env|
-          assert_cache :clean
+          assert_cache :dirty # The cache is shared with the main thread
 
           Post.first
           assert_cache :dirty
@@ -866,12 +855,10 @@ class QueryCacheTest < ActiveRecord::TestCase
         end
       when :clean
         assert connection.query_cache_enabled, "cache should be on"
-        assert_not_nil connection.query_cache
         assert_predicate connection.query_cache, :empty?, "cache should be empty"
       when :dirty
         assert connection.query_cache_enabled, "cache should be on"
-        assert_not_nil connection.query_cache
-        assert_not_predicate connection.query_cache, :empty?, "cache should be dirty"
+        assert_not connection.query_cache.empty?, "cache should be dirty"
       else
         raise "unknown state"
       end
