@@ -20,8 +20,8 @@ class QueryLogsTest < ActiveRecord::TestCase
     ActiveRecord::QueryLogs.prepend_comment = false
     ActiveRecord::QueryLogs.cache_query_log_tags = false
     ActiveRecord::QueryLogs.cached_comment = nil
-    ActiveRecord::QueryLogs.taggings[:application] = -> {
-      "active_record"
+    ActiveRecord::QueryLogs.taggings = {
+      application: -> { "active_record" }
     }
   end
 
@@ -33,7 +33,7 @@ class QueryLogsTest < ActiveRecord::TestCase
     ActiveRecord::QueryLogs.prepend_comment = false
     ActiveRecord::QueryLogs.cache_query_log_tags = false
     ActiveRecord::QueryLogs.clear_cache
-    ActiveRecord::QueryLogs.update_formatter(:legacy)
+    ActiveRecord::QueryLogs.tags_formatter = :legacy
 
     # ActiveSupport::ExecutionContext context is automatically reset in Rails app via an executor hooks set in railtie
     # But not in Active Record's own test suite.
@@ -45,7 +45,8 @@ class QueryLogsTest < ActiveRecord::TestCase
   end
 
   def test_escaping_good_comment_with_custom_separator
-    ActiveRecord::QueryLogs.update_formatter(:sqlcommenter)
+    ActiveRecord::QueryLogs.tags_formatter = :sqlcommenter
+
     assert_equal "app='foo'", ActiveRecord::QueryLogs.send(:escape_sql_comment, "app='foo'")
   end
 
@@ -58,8 +59,8 @@ class QueryLogsTest < ActiveRecord::TestCase
   def test_basic_commenting
     ActiveRecord::QueryLogs.tags = [ :application ]
 
-    assert_sql(%r{select id from posts /\*application:active_record\*/$}) do
-      ActiveRecord::Base.connection.execute "select id from posts"
+    assert_queries_match(%r{select id from posts /\*application:active_record\*/$}) do
+      ActiveRecord::Base.lease_connection.execute "select id from posts"
     end
   end
 
@@ -67,14 +68,14 @@ class QueryLogsTest < ActiveRecord::TestCase
     ActiveRecord::QueryLogs.tags = [ :application ]
     ActiveRecord::QueryLogs.prepend_comment = true
 
-    assert_sql(%r{/\*application:active_record\*/ select id from posts$}) do
-      ActiveRecord::Base.connection.execute "select id from posts"
+    assert_queries_match(%r{/\*application:active_record\*/ select id from posts$}) do
+      ActiveRecord::Base.lease_connection.execute "select id from posts"
     end
   end
 
   def test_exists_is_commented
     ActiveRecord::QueryLogs.tags = [ :application ]
-    assert_sql(%r{/\*application:active_record\*/}) do
+    assert_queries_match(%r{/\*application:active_record\*/}) do
       Dashboard.exists?
     end
   end
@@ -83,7 +84,7 @@ class QueryLogsTest < ActiveRecord::TestCase
     ActiveRecord::QueryLogs.tags = [ :application ]
     record = Dashboard.first
 
-    assert_sql(%r{/\*application:active_record\*/}) do
+    assert_queries_match(%r{/\*application:active_record\*/}) do
       record.destroy
     end
   end
@@ -91,7 +92,7 @@ class QueryLogsTest < ActiveRecord::TestCase
   def test_update_is_commented
     ActiveRecord::QueryLogs.tags = [ :application ]
 
-    assert_sql(%r{/\*application:active_record\*/}) do
+    assert_queries_match(%r{/\*application:active_record\*/}) do
       dash = Dashboard.first
       dash.name = "New name"
       dash.save
@@ -101,7 +102,7 @@ class QueryLogsTest < ActiveRecord::TestCase
   def test_create_is_commented
     ActiveRecord::QueryLogs.tags = [ :application ]
 
-    assert_sql(%r{/\*application:active_record\*/}) do
+    assert_queries_match(%r{/\*application:active_record\*/}) do
       Dashboard.create(name: "Another dashboard")
     end
   end
@@ -109,7 +110,7 @@ class QueryLogsTest < ActiveRecord::TestCase
   def test_select_is_commented
     ActiveRecord::QueryLogs.tags = [ :application ]
 
-    assert_sql(%r{/\*application:active_record\*/}) do
+    assert_queries_match(%r{/\*application:active_record\*/}) do
       Dashboard.all.to_a
     end
   end
@@ -119,12 +120,12 @@ class QueryLogsTest < ActiveRecord::TestCase
     i = 0
     ActiveRecord::QueryLogs.tags = [ { query_counter: -> { i += 1 } } ]
 
-    assert_sql("SELECT 1 /*query_counter:1*/") do
-      ActiveRecord::Base.connection.execute "SELECT 1"
+    assert_queries_match("SELECT 1 /*query_counter:1*/") do
+      ActiveRecord::Base.lease_connection.execute "SELECT 1"
     end
 
-    assert_sql("SELECT 1 /*query_counter:1*/") do
-      ActiveRecord::Base.connection.execute "SELECT 1"
+    assert_queries_match("SELECT 1 /*query_counter:1*/") do
+      ActiveRecord::Base.lease_connection.execute "SELECT 1"
     end
   end
 
@@ -133,34 +134,34 @@ class QueryLogsTest < ActiveRecord::TestCase
     ActiveSupport::ExecutionContext[:temporary] = "value"
     ActiveRecord::QueryLogs.tags = [ temporary_tag: ->(context) { context[:temporary] } ]
 
-    assert_sql("SELECT 1 /*temporary_tag:value*/") do
-      ActiveRecord::Base.connection.execute "SELECT 1"
+    assert_queries_match("SELECT 1 /*temporary_tag:value*/") do
+      ActiveRecord::Base.lease_connection.execute "SELECT 1"
     end
 
     ActiveSupport::ExecutionContext[:temporary] = "new_value"
 
-    assert_sql("SELECT 1 /*temporary_tag:new_value*/") do
-      ActiveRecord::Base.connection.execute "SELECT 1"
+    assert_queries_match("SELECT 1 /*temporary_tag:new_value*/") do
+      ActiveRecord::Base.lease_connection.execute "SELECT 1"
     end
   end
 
   def test_default_tag_behavior
     ActiveRecord::QueryLogs.tags = [:application, :foo]
     ActiveSupport::ExecutionContext.set(foo: "bar") do
-      assert_sql(%r{/\*application:active_record,foo:bar\*/}) do
+      assert_queries_match(%r{/\*application:active_record,foo:bar\*/}) do
         Dashboard.first
       end
     end
-    assert_sql(%r{/\*application:active_record\*/}) do
+    assert_queries_match(%r{/\*application:active_record\*/}) do
       Dashboard.first
     end
   end
 
   def test_connection_is_passed_to_tagging_proc
-    connection = ActiveRecord::Base.connection
+    connection = ActiveRecord::Base.lease_connection
     ActiveRecord::QueryLogs.tags = [ same_connection: ->(context) { context[:connection] == connection } ]
 
-    assert_sql("SELECT 1 /*same_connection:true*/") do
+    assert_queries_match("SELECT 1 /*same_connection:true*/") do
       connection.execute "SELECT 1"
     end
   end
@@ -170,21 +171,23 @@ class QueryLogsTest < ActiveRecord::TestCase
     ActiveSupport::ExecutionContext[:connection] = fake_connection
     ActiveRecord::QueryLogs.tags = [ fake_connection: ->(context) { context[:connection] == fake_connection } ]
 
-    assert_sql("SELECT 1 /*fake_connection:true*/") do
-      ActiveRecord::Base.connection.execute "SELECT 1"
+    assert_queries_match("SELECT 1 /*fake_connection:true*/") do
+      ActiveRecord::Base.lease_connection.execute "SELECT 1"
     end
   end
 
   def test_empty_comments_are_not_added
     ActiveRecord::QueryLogs.tags = [ empty: -> { nil } ]
-    assert_sql(%r{select id from posts$}) do
-      ActiveRecord::Base.connection.execute "select id from posts"
+    assert_queries_match(%r{select id from posts$}) do
+      ActiveRecord::Base.lease_connection.execute "select id from posts"
     end
   end
 
   def test_sql_commenter_format
-    ActiveRecord::QueryLogs.update_formatter(:sqlcommenter)
-    assert_sql(%r{/\*application='active_record'\*/}) do
+    ActiveRecord::QueryLogs.tags_formatter = :sqlcommenter
+    ActiveRecord::QueryLogs.tags = [:application]
+
+    assert_queries_match(%r{/\*application='active_record'\*/}) do
       Dashboard.first
     end
   end
@@ -192,7 +195,7 @@ class QueryLogsTest < ActiveRecord::TestCase
   def test_custom_basic_tags
     ActiveRecord::QueryLogs.tags = [ :application, { custom_string: "test content" } ]
 
-    assert_sql(%r{/\*application:active_record,custom_string:test content\*/}) do
+    assert_queries_match(%r{/\*application:active_record,custom_string:test content\*/}) do
       Dashboard.first
     end
   end
@@ -200,7 +203,7 @@ class QueryLogsTest < ActiveRecord::TestCase
   def test_custom_proc_tags
     ActiveRecord::QueryLogs.tags = [ :application, { custom_proc: -> { "test content" } } ]
 
-    assert_sql(%r{/\*application:active_record,custom_proc:test content\*/}) do
+    assert_queries_match(%r{/\*application:active_record,custom_proc:test content\*/}) do
       Dashboard.first
     end
   end
@@ -211,26 +214,26 @@ class QueryLogsTest < ActiveRecord::TestCase
       { custom_proc: -> { "test content" }, another_proc: -> { "more test content" } },
     ]
 
-    assert_sql(%r{/\*application:active_record,custom_proc:test content,another_proc:more test content\*/}) do
+    assert_queries_match(%r{/\*another_proc:more test content,application:active_record,custom_proc:test content\*/}) do
       Dashboard.first
     end
   end
 
   def test_sqlcommenter_format_value
-    ActiveRecord::QueryLogs.update_formatter(:sqlcommenter)
+    ActiveRecord::QueryLogs.tags_formatter = :sqlcommenter
 
     ActiveRecord::QueryLogs.tags = [
       :application,
       { tracestate: "congo=t61rcWkgMzE,rojo=00f067aa0ba902b7", custom_proc: -> { "Joe's Shack" } },
     ]
 
-    assert_sql(%r{custom_proc='Joe%27s%20Shack',tracestate='congo%3Dt61rcWkgMzE%2Crojo%3D00f067aa0ba902b7'\*/}) do
+    assert_queries_match(%r{custom_proc='Joe%27s%20Shack',tracestate='congo%3Dt61rcWkgMzE%2Crojo%3D00f067aa0ba902b7'\*/}) do
       Dashboard.first
     end
   end
 
   def test_sqlcommenter_format_allows_string_keys
-    ActiveRecord::QueryLogs.update_formatter(:sqlcommenter)
+    ActiveRecord::QueryLogs.tags_formatter = :sqlcommenter
 
     ActiveRecord::QueryLogs.tags = [
       :application,
@@ -241,20 +244,20 @@ class QueryLogsTest < ActiveRecord::TestCase
       },
     ]
 
-    assert_sql(%r{custom_proc='Joe%27s%20Shack',string='value',tracestate='congo%3Dt61rcWkgMzE%2Crojo%3D00f067aa0ba902b7'\*/}) do
+    assert_queries_match(%r{custom_proc='Joe%27s%20Shack',string='value',tracestate='congo%3Dt61rcWkgMzE%2Crojo%3D00f067aa0ba902b7'\*/}) do
       Dashboard.first
     end
   end
 
   def test_sqlcommenter_format_value_string_coercible
-    ActiveRecord::QueryLogs.update_formatter(:sqlcommenter)
+    ActiveRecord::QueryLogs.tags_formatter = :sqlcommenter
 
     ActiveRecord::QueryLogs.tags = [
       :application,
       { custom_proc: -> { 1234 } },
     ]
 
-    assert_sql(%r{custom_proc='1234'\*/}) do
+    assert_queries_match(%r{custom_proc='1234'\*/}) do
       Dashboard.first
     end
   end
@@ -264,7 +267,7 @@ class QueryLogsTest < ActiveRecord::TestCase
     def test_invalid_encoding_query
       ActiveRecord::QueryLogs.tags = [ :application ]
       assert_nothing_raised do
-        ActiveRecord::Base.connection.execute "select 1 as '\xFF'"
+        ActiveRecord::Base.lease_connection.execute "select 1 as '\xFF'"
       end
     end
   end
@@ -273,7 +276,7 @@ class QueryLogsTest < ActiveRecord::TestCase
     ActiveSupport::ExecutionContext[:foo] = "bar"
     ActiveRecord::QueryLogs.tags = [ :application, { custom_context_proc: ->(context) { context[:foo] } } ]
 
-    assert_sql(%r{/\*application:active_record,custom_context_proc:bar\*/}) do
+    assert_queries_match(%r{/\*application:active_record,custom_context_proc:bar\*/}) do
       Dashboard.first
     end
   end
