@@ -114,6 +114,17 @@ class ActiveStorage::VariantTest < ActiveSupport::TestCase
     assert_equal 8, image.height
   end
 
+  test "resized variation of WEBP blob" do
+    blob = create_file_blob(filename: "valley.webp")
+    variant = blob.variant(resize_to_limit: [50, 50]).processed
+    assert_match(/valley\.webp/, variant.url)
+
+    image = read_image(variant)
+    assert_equal "WEBP", image.type
+    assert_equal 50, image.width
+    assert_includes [33, 34], image.height
+  end
+
   test "optimized variation of GIF blob" do
     blob = create_file_blob(filename: "image.gif", content_type: "image/gif")
 
@@ -147,7 +158,7 @@ class ActiveStorage::VariantTest < ActiveSupport::TestCase
   end
 
   test "variation of invariable blob" do
-    assert_raises ActiveStorage::InvariableError do
+    assert_raises ActiveStorage::InvariableError, match: /blob with ID=\d+ and content_type=application\/pdf/ do
       create_file_blob(filename: "report.pdf", content_type: "application/pdf").variant(resize_to_limit: [100, 100])
     end
   end
@@ -246,7 +257,7 @@ class ActiveStorage::VariantTest < ActiveSupport::TestCase
     process_variants_with :mini_magick do
       blob = create_file_blob(filename: "racecar.jpg")
       assert_raise(ActiveStorage::Transformers::ImageProcessingTransformer::UnsupportedImageProcessingArgument) do
-        blob.variant(resize: { "-write": "/tmp/file.erb" }).processed
+        blob.variant(saver: { "-write": "/tmp/file.erb" }).processed
       end
     end
   end
@@ -255,11 +266,11 @@ class ActiveStorage::VariantTest < ActiveSupport::TestCase
     process_variants_with :mini_magick do
       blob = create_file_blob(filename: "racecar.jpg")
       assert_raise(ActiveStorage::Transformers::ImageProcessingTransformer::UnsupportedImageProcessingArgument) do
-        blob.variant(resize: { "something": { "-write": "/tmp/file.erb" } }).processed
+        blob.variant(saver: { "something": { "-write": "/tmp/file.erb" } }).processed
       end
 
       assert_raise(ActiveStorage::Transformers::ImageProcessingTransformer::UnsupportedImageProcessingArgument) do
-        blob.variant(resize: { "something": ["-write", "/tmp/file.erb"] }).processed
+        blob.variant(saver: { "something": ["-write", "/tmp/file.erb"] }).processed
       end
     end
   end
